@@ -1,3 +1,3 @@
-from . import convs, media, widgets
+from . import convs, widgets
 from .fields import Field, FieldSet, FieldList, FileField, FieldBlock
 from .form import Form
