@@ -52,8 +52,9 @@ class Template(object):
                 template_type = ext[1:]
                 if template_type in self.engines:
                     return file_name[len(d)+1:], self.engines[template_type]
-        raise TemplateError('Template or engine for template "%s" not found. Dirs %r' % \
-                            (pattern, self.dirs))
+        raise TemplateError(
+                'Template or engine for template "{}" not found in '\
+                'directories {!r}'.format(pattern, self.dirs))
 
 
 class BoundTemplate(object):
@@ -109,12 +110,3 @@ class BoundTemplate(object):
         resp = self.render(template_name, __data)
         return Response(resp,
                         content_type=content_type)
-
-
-def render_to(self, template_name):
-    @request_filter
-    def render_to(env, data, next_handler):
-        data.env = env
-        return Response(env.template.render(template_name, **data))
-    return render_to
-
