@@ -3,17 +3,18 @@
 __all__ = ['match', 'method', 'static_files', 'prefix', 
            'subdomain', 'namespace', 'by_method']
 
+import six
 import logging
 import os
 from os import path
-import posixpath
-from urllib import unquote
-from webob.exc import HTTPMethodNotAllowed
+from six.moves.urllib.parse import unquote
+from webob.exc import HTTPMethodNotAllowed, HTTPNotFound
 from webob.static import FileApp
 from .core import WebHandler, cases
 from . import Response
 from .url_templates import UrlTemplate
 from .reverse import Location
+from iktomi.utils.deprecation import deprecated
 
 
 logger = logging.getLogger(__name__)
@@ -39,11 +40,19 @@ class match(WebHandler):
                   convs={'date': MyDateUrlConv}) | by_date,
     '''
 
-    def __init__(self, url='', name='', convs=None):
+
+    def __init__(self, url='', name='', fragment=None, convs=None):
         self.url = url
         self.url_name = name
+        self.fragment = fragment
         self.builder = UrlTemplate(url, converters=convs)
 
+        self.fragment = fragment
+        if self.fragment is not None:
+            self.fragment_builder = UrlTemplate(self.fragment, converters=convs)
+        else:
+            self.fragment_builder = None
+
     def match(self, env, data):
         matched, kwargs = self.builder.match(env._route_state.path, env=env)
         if matched is not None:
@@ -54,11 +63,13 @@ class match(WebHandler):
     __call__ = match # for beautiful tracebacks
 
     def _locations(self):
-        return {self.url_name: (Location(self.builder), {})}
+        location = Location(self.builder,
+                            fragment_builder=self.fragment_builder)
+        return {self.url_name: (location, {})}
 
     def __repr__(self):
-        return '%s(\'%s\', \'%s\')' % \
-                (self.__class__.__name__, self.url, self.url_name)
+        return '{}({!r}, {!r})'.format(self.__class__.__name__,
+                                       self.url, self.url_name)
 
 
 class prefix(WebHandler):
@@ -81,6 +92,7 @@ class prefix(WebHandler):
     '''
 
     def __init__(self, _prefix, convs=None, name=None):
+        self.url = _prefix
         self.builder = UrlTemplate(_prefix, match_whole_str=False, 
                                    converters=convs)
         if name is not None:
@@ -105,7 +117,7 @@ class prefix(WebHandler):
         return locations
 
     def __repr__(self):
-        return '%s(\'%r\')' % (self.__class__.__name__, self.builder)
+        return '{}({!r})'.format(self.__class__.__name__, self.url)
 
 
 class namespace(WebHandler):
@@ -138,8 +150,9 @@ class namespace(WebHandler):
         builders = []
         while all_locations and all_locations[0].builders:
             builder = all_locations[0].builders[0]
-            if any(x.builders and x.builders[0] is builder
-                   for x in all_locations[1:]):
+            if len(all_locations) > 1 and \
+                not any(not (x.builders and x.builders[0] is builder)
+                        for x in all_locations[1:]):
                 builders.append(builder)
                 for loc in all_locations:
                     loc.builders.pop(0)
@@ -167,6 +180,7 @@ class namespace(WebHandler):
         for loc in locations.values():
             loc[0].builders = builders
             loc[0].subdomains = subdomains
+
         return locations
 
 
@@ -196,7 +210,7 @@ class method(WebHandler):
     __call__ = method
 
     def __repr__(self):
-        return 'method(%s)' % ', '.join(repr(n) for n in self._names)
+        return 'method({})'.format(', '.join(repr(n) for n in self._names))
 
 
 class by_method(cases):
@@ -210,7 +224,7 @@ class by_method(cases):
     def __init__(self, handlers_dict, default_handler=None):
         handlers = []
         for methods, handler in handlers_dict.items():
-            if isinstance(methods, basestring):
+            if isinstance(methods, six.string_types):
                 methods = (methods,)
             handlers.append(method(*methods, strict=False) | handler)
         if default_handler is not None:
@@ -246,9 +260,7 @@ class subdomain(WebHandler):
     '''
 
     def __init__(self, *subdomains, **kwargs):
-        # XXX is this convert to unicode actually needed?
-        self.subdomains = [unicode(x) if x is not None else None
-                           for x in subdomains]
+        self.subdomains = subdomains
 
         # this attribute is used for ducktyping in Location, be careful
         self.primary = kwargs.pop('primary', self.subdomains[0])
@@ -259,7 +271,7 @@ class subdomain(WebHandler):
             # A shortcut for subdomain(..) | namespace(name)
             self._next_handler = namespace(name)
 
-        if kwargs:
+        if kwargs: # pragma: no cover
             raise TypeError("subdomain.__init__ got an unexpected keyword "
                             "arguments {}".format(",".join(kwargs)))
 
@@ -292,8 +304,7 @@ class subdomain(WebHandler):
         return locations
 
     def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, self.subdomains)
-
+        return '{}({!r})'.format(self.__class__.__name__, self.subdomains)
 
 
 class static_files(WebHandler):
@@ -307,12 +318,14 @@ class static_files(WebHandler):
         self.location = location
         self.url = url
 
-    def construct_reverse(self):
-        def url_for_static(part):
-            while part.startswith('/'):
-                part = part[1:]
-            return path.join(self.url, part)
-        return url_for_static
+    def url_for_static(self, part):
+        while part.startswith('/'):
+            part = part[1:]
+        return path.join(self.url, part)
+
+    @deprecated("Use static.url_for_static instead")
+    def construct_reverse(self): # pragma: no cover
+        return self.url_for_static
 
     def translate_path(self, pth):
         """Translate a /-separated PATH to the local filename syntax."""
@@ -336,6 +349,8 @@ class static_files(WebHandler):
     def static_files(self, env, data):
         path_info = unquote(env.request.path)
         if path_info.startswith(self.url):
+            if path_info.endswith('/'):
+                raise HTTPNotFound
             file_path = self.translate_path(path_info[len(self.url):])
             if file_path and path.exists(file_path) and path.isfile(file_path):
                 return FileApp(file_path)
@@ -347,6 +362,6 @@ class static_files(WebHandler):
     __call__ = static_files
 
     def __repr__(self):
-        return '%s(\'%r\', \'%r\')' % (self.__class__.__name__, 
+        return '{}({!r}, {!r})'.format(self.__class__.__name__, 
                                        self.location, self.url)
 
